#-------------------------------------------------------------------------------
# Name:        Axeda Thermostat Cloud Demo
# Purpose:     To illistrait how to use the Axeda Cloud AT commands to
#              implement a basic thermostat.
#
# Author:      sep
#
# Created:     27/04/2014
# Copyright:   (c) Inventek Systems, LLC. 2014
#
# Name:        INVENTEK WIRELESS INTEROPERABILITY NETWORK (IWIN), AT Command Software
# Purpose:     To provide Inventek customers Python scripts illustrating how to
#              use the IWIN AT Command Software.
#
#------------------------------------------------------------------------------
#LICENSE:
#   THE IWIN PROGRAM IS FREE SOFTWARE FOR INVENTEK CUSTOMERS ONLY.
#   INVENTEK SYSTEMS RETAINS ALL OWNERSHIP AND INTELLECTUAL PROPERTY RIGHTS
#   IN THE IWIN CODE ACCOMPANYING THIS MESSAGE AND IN ALL DERIVATIVES HERETO.
#   USER MAY USE THE IWIN CODE, AND ANY DERIVATIVES CREATED BY ANY PERSON OR
#   ENTITY BY OR ON USER?S BEHALF, EXCLUSIVELY WITH INVENTEK'S PROPRIETARY
#   PRODUCTS.
#
#   USER AGREES THAT USER IS SOLELY RESPONSIBLE FOR TESTING THE CODE AND
#   DETERMINING IT?S SUITABILITY.  INVENTEK HAS NO OBLIGATION TO MODIFY, TEST,
#   CERTIFY, OR SUPPORT THE CODE.
#
#   USER ACCEPTANCE AND/OR USE OF THE IWIN CODE CONSTITUTES AGREEMENT TO THE
#   TERMS AND CONDITIONS OF THIS NOTICE. IWIN CAN BE REDISTRIBUTED AND/OR
#   MODIFIED BY INVENTEK CUSTOMERS ONLY AND UNDER THE TERMS OF THE GNU GENERAL
#   PUBLIC LICENSE AS PUBLISHED BY THE FREE SOFTWARE FOUNDATION, EITHER VERSION
#   3 OF THE LICENSE, OR (AT USER?S OPTION) ANY LATER VERSION.
#
#   THIS PROGRAM IS DISTRIBUTED IN THE HOPE THAT IT WILL BE USEFUL, BUT WITHOUT
#   ANY WARRANTY; WITHOUT EVEN THE IMPLIED WARRANTY OF MERCHANTABILITY OR FITNESS
#   FOR A PARTICULAR PURPOSE.  REFERENCE THE GNU GENERAL PUBLIC LICENSE FOR MORE
#   DETAILS.
#
#   CODE ACCOMPANYING THIS MESSAGE IS SUPPLIED BY INVENTEK "AS IS".  NO WARRANTIES,
#   WHETHER EXPRESSED, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED TO,
#   IMPLIED WARRANTIES OF NON-INFRINGEMENT, MERCHANTABILITY AND FITNESS FOR A
#   PARTICULAR PURPOSE APPLY TO THIS CODE, IT?S INTERACTION WITH INVENTEK 'S
#   PRODUCTS, COMBINATION WITH ANY OTHER PRODUCTS, OR USE IN ANY APPLICATION.
#
#   USER ACKNOWLEDGES AND AGREES THAT, IN NO EVENT, SHALL INVENTEK BE LIABLE,
#   WHETHER IN CONTRACT, WARRANTY, TORT (INCLUDING NEGLIGENCE OR BREACH OF
#   STATUTORY DUTY), STRICT LIABILITY, INDEMNITY, CONTRIBUTION, OR OTHERWISE,
#   FOR ANY INDIRECT, SPECIAL, PUNITIVE, EXEMPLARY, INCIDENTAL OR CONSEQUENTIAL
#   LOSS, DAMAGE, FOR COST OR EXPENSE OF ANY KIND WHATSOEVER RELATED TO THE CODE,
#   HOWSOEVER CAUSED, EVEN IF INVENTEK HAS BEEN ADVISED OF THE POSSIBILITY OR THE
#   DAMAGES ARE FORESEEABLE.  TO THE FULLEST EXTENT ALLOWABLE BY LAW, INVENTEK 'S
#   TOTAL LIABILITY ON ALL CLAIMS IN ANY WAY RELATED TO THIS CODE, SHALL NOT
#   EXCEED THE PRICE PAID DIRECTLY TO INVENTEK SPECIFICALLY TO HAVE THIS CODE
#   DEVELOPED.
#
#   USER SHOULD HAVE RECEIVED A COPY OF THE GNU GENERAL PUBLIC LICENSE ALONG
#   WITH THIS PROGRAM. IF NOT, SEE <HTTP://WWW.GNU.ORG/LICENSES/>.
#-------------------------------------------------------------------------------
#
# Version:      0.0         sep         20140427    Intial release
#               1.0         sep         20140506    Added SoftAP, 2-button exit, corrections
#               1.1         sep         20140810    Update license
#-------------------------------------------------------------------------------

import math, sys, serial, serial.tools.list_ports, time

#Quiet mode, 0 = Quiet, 1 = Verbose
quiet = 0

#Send timeout
timeout = 30


#Send a command to a eS-WiFi module and wait for prompt("\r\n> ") responce
def sendCMDBlocking(serPort,cmd):
    rcvString =""

    #Send Command
    if (quiet == 1):
        sys.stdout.write(cmd)
    serPort.write(cmd + '\r')

    st = time.time()                    #Get time

    #read until "\r\n> " received
    found = 0
    while found != 1:
        for c in serPort.read():
            rcvString += c
            if  '\r\n> ' in rcvString:
                found = 1
            #Endif
        #End for

        #Check for timeout
        if (time.time() - st) > timeout:
            return '-1'                 #Timeout error
        #Endif
    #End while

    #Show responce
    if (quiet == 1):
        sys.stdout.write(rcvString)

    #Get payload if OK responce
    if 'OK\r\n> ' in rcvString:
        rcvString=rcvString[2:(len(rcvString)-8)]
    #Endif

    #Return payload or Error
    return rcvString


#Send CMD and wait for responce, waiting nsec seconds before timing out
def sendCMDWait(serPort,cmd,responce,nsec):
    rcvString =""

    #Send Command
    if (quiet == 1):
        sys.stdout.write(cmd)
    serPort.write(cmd + '\r')

    st = time.time()                    #Get time

    #Check each message recieved for responce
    found = 0
    while found != 1:

        #read until "\r\n> " received
        found0 = 0
        while found0 != 1:
            for c in serPort.read():
                rcvString += c
                #if (quiet == 1):
                sys.stdout.write(c)
                if  "\r\n> " in rcvString:
                    found0 = 1
                #Endif
            #End for

            #Check for timeout
            if (time.time() - st) > nsec:
                return '-1'                 #Timeout error
            #Endif
        #End while

        #Show responce
        if (quiet == 1):
            sys.stdout.write(rcvString)

        #Check for responce
        if  responce in rcvString:
            found = 1
        else:
            rcvString = ""                  #Clear string
        #End if
    #End while

    #Show responce
    if (quiet == 1):
        sys.stdout.write(rcvString)

    #Get payload if OK responce
    if 'OK\r\n> ' in rcvString:
        rcvString=rcvString[2:(len(rcvString)-8)]
    #Endif

    #Return payload or Error
    return rcvString


#Search COM port for and eS-WiFi module
def findModule(timeout):
    rcvString =""

    #Get ports
    portList = list(serial.tools.list_ports.comports())
    #print(portList)

    #Search for module
    for index, openPort in enumerate(portList):

        #Serial Port Setup
        port = portList[index][0]
        baud = 115200
        serPort = serial.Serial(port,baud)
        serPort.timeout = 10

        serPort.write('\r')                 #Send CR

        st = time.time()                    #Get time

        #read until "\r\n> " received
        found = 0
        while found != 1:
            for c in serPort.read():
                rcvString += c
                if  '\r\n> ' in rcvString:
                    found = 1
                #Endif
            #End for

            #Check for timeout
            if (time.time() - st) > timeout:
                return '-1'                 #Timeout error
            #Endif
        #End while

        #Try to get info
        rcv = sendCMDBlocking(serPort,"I?")

        #Close serial port
        serPort.close()

        #Found?
        if (found == 1):
            #print("Port = " + port)
            if "ISM4" in rcv:
                print(rcv)
                return port
            #Endif
        #End if
    #End for
    return -1

#Application
def main():

    #Redirect std err to std out
    sys.stderr = sys.stdout

    #Serial Port Setup
    port = findModule(10)
    if (port == -1):
        print( "Failed to find an eS-WiFi module!")
        sys.exit(-1)
    #End if
    baud = 115200
    ser = serial.Serial(port,baud)
    ser.timeout = 10

    #Get Firmware information
    if (quiet == 1):
        rcv = sendCMDBlocking(ser,"I?")
        print(rcv)
    #End if

    #Setup----------------------------------------------------------------------
    mCnt = 50                   #Monitoring Count
    cal = 50                    #Temperature Calibration
    dCnt = 15                   #Delay seconds

    setPointF = 680.0                       #Set point deg F
    setPoint = (setPointF - 320) * (1/1.8)  #Set point deg C
    setLow = setPoint - 11.0                #Low event point
    setHigh = setPoint + 11.0               #High event point
    limitLow = setPoint - 44.0              #Low alarm limit
    limitHigh = setPoint + 44.0             #High alarm point

    alarmCount = 0              #Alarm counter
    dataCount = 0               #Data counter
    eventCount = 0              #Event counter
    locatCount = 0              #Location counter
    monitorCount = 0            #Monitor counter
    pointCount = 0              #Set point counter
    #---------------------------------------------------------------------------

    minTmp = 999                #Minimum temperature
    maxTmp = -999               #Maximum temperature
    aveTmp = 0                  #Average temperature
    totalTmp = 0                #Total temperature

    deg = unichr(0xB0)          #Degree character
    ctrlQ = unichr(0x11)        #Ctrl-Q character
    exitApp = 0

    Button0 = '1'               #Up button
    Button1 = '1'               #Down button

    sn = ""                     #Serial Number
    useSN = 1                   #0 = MAC, 1 = Serial Number
    useSoftAP = 1               #Join using, 0 = Cx commands, 1 = A0 (SoftAP)

    #Start Application
    print( "Axeda Thermostat Cloud Demo:" )
    print( "Reporting cycle:        " + str(dCnt) + " seconds" )
    print( "Temperature Limit High: " + ("%.0F" % limitHigh) + " (" + ("%.1F" % (limitHigh/10)) + deg +"C/" + ("%.1F" % ((limitHigh*0.18)+32)) + deg + "F)" )
    print( "Temperature Set High:   " + ("%.0F" % setHigh) + " (" + ("%.1F" % (setHigh/10)) + deg +"C/" + ("%.1F" % ((setHigh*0.18)+32)) + deg + "F)" )
    print( "Temperature Set Point:  " + ("%.0F" % setPoint) + " (" + ("%.1F" % (setPoint/10)) + deg +"C/" + ("%.1F" % ((setPoint*0.18)+32)) + deg + "F)" )
    print( "Temperature Set Low:    " + ("%.0F" % setLow) + " (" + ("%.1F" % (setLow/10)) + deg +"C/" + ("%.1F" % ((setLow*0.18)+32)) + deg + "F)" )
    print( "Temperature Limit Low:  " + ("%.0F" % limitLow) + " (" + ("%.1F" % (limitLow/10)) + deg +"C/" + ("%.1F" % ((limitLow*0.18)+32)) + deg + "F)" )
    print( "Cycle Limit:            " + str(mCnt) )
    print( "Communications:         " + ser.portstr + "@" + str(baud))
    print( "" )


    #Join Network
    if (useSoftAP == 0):
        rcv = sendCMDBlocking(ser, "C1=ssid")                   #Set SSID
        rcv = sendCMDBlocking(ser, "C2=password")               #Set Password
        rcv = sendCMDBlocking(ser, "C3=4")                      #WPA-2 Mixed
        rcv = sendCMDBlocking(ser, "C4=1")                      #Use DHCP

        #Get start time
        st = time.time()

        #Join
        rcv = sendCMDBlocking(ser, "C0")                        #Join network
        if (rcv != '-1'):
            if "ERROR" in rcv:                                  #Check for error
                print( "Join failed to connect to network")
                sys.exit(-1)
            else:
                print( "Time: " + ("%012.3F" % (time.time()-st)) + " Joining: " + rcv[10:] )
            #End if
        else:
            print( "Joined timed out" )
            sys.exit(-1)
        #End if
    else:
        #SoftAP
        rcv = sendCMDBlocking(ser, "Z5")                        #Get MAC address
        print( "Starting Configuration Access Point, look for Axeda_" + rcv )

        #Get start time
        st = time.time()

        #Setup SoftAP
        rcv = sendCMDBlocking(ser, "AS=1,Axeda")                #Set SoftAP SSID

        #Start and wait for Join message
        rcv = sendCMDWait(ser, "A0","[JOIN   ]",300)            #Start SoftAP
        if (rcv != '-1'):                                       #Check for timeout
            if "ERROR" in rcv:                                  #Check for error
                print( "Join failed to connect to network")
                sys.exit(-1)
            else:
                print( "Time: " + ("%012.3F" % (time.time()-st)) + " Joining: " + rcv[10:] )
            #End if
        else:
            print( "Timeout, failed to connect to a network")
            serPort.write(ctrlQ)                                #Send Quit
            sys.exit(-1)
        #End if
    #End if

    #Serial Number
    if (useSN == 1):
        #Use Serial Number
        sn = sendCMDBlocking(ser, "ZS")         #Get Serial Number
    else:
        #Use MAC as serial number
        sn = sendCMDBlocking(ser, "Z5")         #Get MAC
        sn = sn.replace(':','')                 #Remove ':'
    #Endif

    #Configure LEDs
    rcv = sendCMDBlocking(ser, "G4=3,1")        #Red LED
    rcv = sendCMDBlocking(ser, "G4=4,1")        #Green LED

    #Green - OFF, RED - OFF
    rcv = sendCMDBlocking(ser, "G3=0,1,0")      #Red LED
    rcv = sendCMDBlocking(ser, "G3=1,1,0")      #Green LED

    #Register with Axeda Cloud
    print( "Time: " + ("%012.3F" % (time.time()-st)) + " Registering SN: " + sn )
    rcv = sendCMDBlocking(ser, "K1=dev6-connect.axeda.com,10101,Inventek,"+ sn)     #Settings
    #rcv = sendCMDBlocking(ser, "K1=toolbox-stage-connect.axeda.com,80,nlr_inventek,nlr__tmcdaniel_axeda_com___06059860")     #Settings
    rcv = sendCMDBlocking(ser, "K1")            #Register
    if 'Error' in rcv:
        print( "Registration failed: \r\n" + rcv )
    rcv = sendCMDBlocking(ser, "K1=?")          #Get settings
    print( " Settings: " + rcv )

    #Send Location to Axeda Cloud
    lat = 42.557399
    lon = -71.299377
    alt = 250
    print( "Time: " + ("%012.3F" % (time.time()-st)) + " Location: Lat = " + ("%.6F" % lat) + " Lon = " + ("%.6F" % lon) + " Alt = " + str(alt) )
    rcv = sendCMDBlocking(ser, "K2=3," + ("%.6F" % lat) +"," + ("%.6F" % lon) + "," + str(alt))
    locatCount += 1

    #Send Event Monitoring Start to Axeda Cloud
    print( "Time: " + ("%012.3F" % (time.time()-st)) + " Event, Monitoring, Started" )
    rcv = sendCMDBlocking(ser, "K2=2,Monitoring,Started")
    eventCount += 1

    #Send Set Point to Axeda CLOUD
    print( "Time: " + ("%012.3F" % (time.time()-st)) + " Set Point:   " + ("%.0F" % setPoint) + " (" + ("%.1F" % (setPoint/10.0)) + deg +"C/" + ("%.1F" % ((setPoint*0.18)+32)) + deg + "F)" )
    rcv = sendCMDBlocking(ser, "K2=1,Set Point,1," + ("%.0F" % setPoint))
    pointCount += 1


    #Reporting Loop
    while monitorCount < mCnt:
        #Get Thermistor value
        threm = sendCMDBlocking(ser, "G2=0,5")      #Read ADC0

        #Calculate temperature
        thremo_res =  43000.0 / (( 4096.0 / float(threm)) - 1 )
        logval = math.log( thremo_res / 10000.0 )
        tmp = 1.0 / ( logval / 3380.0 + 1.0 /298.15 ) -273.15

        #Calibrate temperature
        calTmp = (tmp * 10) - cal

        #Send Temperature to Axeda Cloud
        print( "Time: " + ("%012.3F" % (time.time()-st)) + " Temperature: " + ("%.0F" % calTmp) + " (" + ("%.1F" % (calTmp/10)) + deg + "C/" + ("%.1F" % ((calTmp*0.18)+32)) + deg + "F)"  + " Cnt: " + str(monitorCount+1))
        rcv = sendCMDBlocking(ser, "K2=1,Temperature,1," + ("%.0F" % calTmp))
        dataCount += 1


        #Stats
        if (calTmp < minTmp):
            minTmp = calTmp             #Set new minimum
        if (calTmp > maxTmp):
            maxTmp = calTmp             #Set new maximum
        totalTmp += calTmp              #Sum temperature


        #Alarm Check
        if (calTmp + 0.5 >= limitHigh):
            print( "Time: " + ("%012.3F" % (time.time()-st)) + " Alarm, Limit High reached" )
            rcv = sendCMDBlocking(ser, "K2=0,Temperature,Over Limit,100")
            alarmCount += 1
        elif (calTmp - 0.5 <= limitLow):
            print( "Time: " + ("%012.3F" % (time.time()-st)) + " Alarm, Limit Low reached" )
            rcv = sendCMDBlocking(ser, "K2=0,Temperature,Under Limit,100")
            alarmCount += 1
        #Endif


        #Events
        if (calTmp + 0.5 >= setHigh):
            #Green - OFF, RED - ON
            rcv = sendCMDBlocking(ser, "G3=0,1,1")      #Red LED
            rcv = sendCMDBlocking(ser, "G3=1,1,0")      #Green LED

            #Send event to Axeda Cloud
            print( "Time: " + ("%012.3F" % (time.time()-st)) + " Event, Set High reached" )
            rcv = sendCMDBlocking(ser, "K2=2,Monitoring,Set High")
            eventCount += 1
        elif (calTmp - 0.5 <= setLow):
            #Green - OFF, RED - ON
            rcv = sendCMDBlocking(ser, "G3=0,1,1")      #Red LED
            rcv = sendCMDBlocking(ser, "G3=1,1,0")      #Green LED

            #Send event to Axeda Cloud
            print( "Time: " + ("%012.3F" % (time.time()-st)) + " Event, Set Low reached" )
            rcv = sendCMDBlocking(ser, "K2=2,Monitoring,Set Low")
            eventCount += 1
        else:
            #Green - ON, RED - OFF
            rcv = sendCMDBlocking(ser, "G3=0,1,0")      #Red LED
            rcv = sendCMDBlocking(ser, "G3=1,1,1")      #Green LED
        #Endif


        #Loop Delay
        for j in range(1,((dCnt-2)*8)):
            time.sleep(.125)                             #125ms

            #Button Events
            Button0 = sendCMDBlocking(ser, "G2=0,3")    #Set point up (GPIO0)
            Button1 = sendCMDBlocking(ser, "G2=1,3")    #Set point down (GPIO1)

            #Press both buttons to exit
            if ( ('0' in Button0) and ('0' in Button1) ) and (exitApp != 1):
                exitApp = 1
                print( "Time: " + ("%012.3F" % (time.time()-st)) + " User request, exiting" )
                Button0 = '1'                           #Clear button
                Button1 = '1'                           #Clear button
                cycles = monitorCount + 1               #Save count
                monitorCount = mCnt                     #Move count to limit
            #End if

            #Check Up Button
            if ('0' in Button0) and (exitApp != 1):
                #increment set point
                setPoint += 5.5

                #Send event to Axeda Cloud
                print( "Time: " + ("%012.3F" % (time.time()-st)) + " Event, Set Point, UP" )
                rcv = sendCMDBlocking(ser, "K2=2,Set Point,UP")
                eventCount += 1
            #Endif

            #Check Down button
            if ('0' in Button1):
                #decrement set point
                setPoint -= 5.5

                #Send event to Axeda Cloud
                print( "Time: " + ("%012.3F" % (time.time()-st)) + " Event, Set Point, DOWN" )
                rcv = sendCMDBlocking(ser, "K2=2,Set Point,DOWN")
                eventCount += 1
            #Endif

            #Update setting if either button pressed
            if ( ('0' in Button0) or ('0' in Button1) and (exitApp != 1) ):
                #Calculate limits
                setLow = setPoint - 11.0
                setHigh = setPoint + 11.0
                limitLow = setPoint - 44.0
                limitHigh = setPoint + 44.0

                #Send Set Point to Axeda CLOUD
                print( "Time: " + ("%012.3F" % (time.time()-st)) + " Set Point:   " + ("%.0F" % setPoint) + " (" + ("%.1F" % (setPoint/10.0)) + deg + "C/" + ("%.1F" % ((setPoint*0.18)+32)) + deg + "F)" )
                rcv = sendCMDBlocking(ser, "K2=1,Set Point,1," + ("%.0F" % setPoint))
                pointCount += 1
            #Endif
        #End for

        monitorCount += 1                       #Monitor counter
    #End while

    #Send Monitoring Stopped to Axeda Cloud
    print( "Time: " + ("%012.3F" % (time.time()-st)) + " Event, Monitoring, Stopped" )
    rcv = sendCMDBlocking(ser, "K2=2,Monitoring,Stopped")
    eventCount += 1


    #Leave Network
    rcv = sendCMDBlocking(ser, "CD")            #Leave network

    #Green - OFF, RED - OFF
    rcv = sendCMDBlocking(ser, "G3=0,1,0")      #Red LED
    rcv = sendCMDBlocking(ser, "G3=1,1,0")      #Green LED

    #Close serial port
    ser.close()


    #Statistics
    aveTmp = totalTmp / dataCount
    print( "" )
    print( "Summary:" )
    print( "Minimum Temperature: " + ("%.0F" % minTmp) + " (" + ("%.1F" % (minTmp/10)) + deg + "C/" + ("%.1F" % ((minTmp*0.18)+32)) + deg + "F)" )
    print( "Average Temperature: " + ("%.0F" % aveTmp) + " (" + ("%.1F" % (aveTmp/10)) + deg + "C/" + ("%.1F" % ((aveTmp*0.18)+32)) + deg + "F)" )
    print( "Maximum Temperature: " + ("%.0F" % maxTmp) + " (" + ("%.1F" % (maxTmp/10)) + deg + "C/" + ("%.1F" % ((maxTmp*0.18)+32)) + deg + "F)" )
    print( "" )
    print( "Alarm Count:         " + str(alarmCount) )
    print( "Data Count:          " + str(dataCount + pointCount) )
    print( "Event Count:         " + str(eventCount) )
    print( "Location Count:      " + str(locatCount) )
    print( "" )
    if (exitApp == 0):
        print( "Total Cycles:        " + str(monitorCount) )
    else:
        print( "Total Cycles:        " + str(cycles) )

if __name__ == '__main__':
    sys.exit(main())

